/*
  MQTT-Controller für KLF200
  Destination: ESP8266 Controller
  v1.0.4 built on 20.11.2024

  MQTT-Subscriptions:
  velux/set/dimmtime
  velux/set/scene
  velux/set/ch1 ... velux/set/ch8
  velux/set/velux1 ... velux/set/velux4

  D0 GPIO16  -> KLF200 Input 1 (high at boot)
  D1 GPIO5   -> KLF200 Input 2
  D2 GPIO4   -> KLF200 Input 3
  D3 GPIO0   -> KLF200 Input 4 (high at boot, connected to Flash-button)
  D4 GPIO2   -> KLF200 Input 5 LED / (high at boot))
  D5 GPIO14  -> KLF200 Input 6
  D6 GPIO12  -> KLF200 Input 7 or Dallas DS18B20 or DHT11 Sensor
  D7 GPIO13  -> KLF200 Input 8 if no Dallas and no DHT11
  D8 GPIO15  -> n/a - difficult as output (must be low at boot!)
  RxD GPIO3  -> KLF200 Input 9 or RxD to PC_DIMMER (high at boot)
  TxD GPIO1  -> KLF200 Input 10 or TxD from PC_DIMMER (high at boot)
  GND
  3V3
  A0         -> only Analog Input

  GPIOs 0, 2 and 15 control the boot of ESP8266:
  15 0 2 Mode   Description
  ==================================================
   L H H  Flash  Boot from SPI Flash (Normal running) 
   L L H  UART   Program via UART (Tx/Rx)
   H x x  SDIO   Boot from SD-Card

  GPIO9 and GPIO10 are reserved for flash-chip but could be used as
  GPIOs by reconfiguring the flash-communication

  As GPIO15 is wired to ESP8266 Flash to boot you have to put a
  4k7 resistor between GPIO15 and GND to let the ESP8266 boot correctly!
*/

#include "vlx2mqtt.h"

void TimerSecondsFcn() {
  // counter for MQTT-reconnection
  if (mqttReconnectTimeout > 0) {
    mqttReconnectTimeout -= 1;
  }

  MQTT_publish();
}

void Timer100msFcn() {
  for (int ch=0; ch<lastVeluxChannel; ch++) {
    if (shutterDisableCounter[ch] > 0) {
      shutterDisableCounter[ch] -= 1;

      if (shutterDisableCounter[ch] == 0) {
        // disable signal-contact
        veluxSetChannel(ch + 1, false);
      }
    }
  }
}

void setup() {
  pinMode(D0, OUTPUT);
  pinMode(D1, OUTPUT);
  pinMode(D2, OUTPUT);
  pinMode(D3, OUTPUT);
  pinMode(D4, OUTPUT);
  pinMode(D5, OUTPUT);
  #if lastVeluxChannel >= 8
    pinMode(D6, OUTPUT);
    pinMode(D7, OUTPUT);
  #endif
  #if lastVeluxChannel == 10
    pinMode(3, OUTPUT); // RxD-Pin
    pinMode(1, OUTPUT); // TxD-Pin
  #endif

  // turn-off all KLF-channels
  for (int ch=1; ch<=lastVeluxChannel; ch++) {
    veluxSetChannel(ch, false);
  }

  #if USE_PCDIMMER == 1
    Serial.begin(115200); // used for controlling R4-Lights
  #endif
  delay(1000);

  // Start the HTTP-Server
  webserver_init();

  // publish DNS-name
  MDNS.begin(host);
  MDNS.addService("http", "tcp", 80);

  // configure MQTT-client
  mqttclient.setServer(mqtt_server, mqtt_serverport);
  mqttclient.setCallback(MQTT_callback);

  wifi_installhandlers();

  #if USE_DALLAS == 1
    // start OneWire-DS18B20 sensors
    // init temperature sensors
    temperatureSensors.begin(NonBlockingDallas::resolution_12, 1500);
    temperatureSensors.onIntervalElapsed(handleDallasIntervalElapsed);
    //temperatureSensors.onTemperatureChange(handleDallasTemperatureChange);
    //temperatureSensors.onDeviceDisconnected(handleDallasDeviceDisconnected);
  #endif

  // Initiate the timers
  TimerSeconds.attach_ms(1000, TimerSecondsFcn);
  Timer100ms.attach_ms(100, Timer100msFcn);
}

void loop() {
  if (WiFiAvailable) {
    // network is connected... do something

    // handle MQTT-transmission
    MQTT_loop();

    // handle webserver on port 80
    webserver.handleClient();

    // update DNS-configuration
    MDNS.update();

    #if USE_PCDIMMER == 1
      // check for new data in uart-buffer from PC_DIMMER Wandpanel
      if (Serial.available() > 0) {
        pcdimmerReadUart(Serial.read());
      }
    #endif
  }else{
    wifi_init(); // (re)connect to WiFi
  }
}

#if USE_DALLAS == 1
  void handleDallasIntervalElapsed(int deviceIndex, int32_t temperatureRAW)
  {
    // we expect only one sensor, so ignore the deviceIndex for now
    temperature = temperatureSensors.rawToCelsius(temperatureRAW);
  }
#endif
